uniform sampler2D videoFrame;
uniform vec3 addParams;
varying highp vec2 textureCoordinate;

uniform vec2 leftEye;
uniform vec2 mouth;
uniform vec4 faceRect;
uniform vec2 rightEye;

vec3 vecMult(vec3 color, float kernelVal) {
    return color * kernelVal;
}

vec3 mild() {
    vec3 color;
    float gray1, gray2;
    float delta = 1.0 / 600.0;
    color = texture2D(videoFrame, textureCoordinate).rgb;
    gray1 = dot(color, vec3(0.299, 0.587, 0.114));
    vec3 tmpSum = vec3(0.0);
    for (highp int i = 0; i < 5; i++){
        for (highp int j = 0; j < 5; j++){
            float x, y;
            x = float(i - 2);
            y = float(j - 2);
            vec3 tmp = texture2D(videoFrame, textureCoordinate + vec2(delta * x, delta * y)).rgb;
            gray2 = dot(tmp, vec3(0.299, 0.587, 0.114));
            float grayDelta = abs(gray1 - gray2);
            if (grayDelta < 0.05) {
                tmpSum += vecMult(tmp, 0.04);
            } else if (grayDelta < 0.10) {
                tmpSum += vecMult(tmp, 0.02);
                tmpSum += vecMult(color, 0.02);
            } else {
                tmpSum += vecMult(color, 0.04);
            }
        }
    }
    vec4 base = vec4(tmpSum, 1.0);
    vec3 overlay;
    overlay.r = sqrt(tmpSum.r);
    overlay.g = sqrt(tmpSum.g);
    overlay.b = sqrt(tmpSum.b);
    return mix(tmpSum.rgb, overlay.rgb, 0.3);
}

 vec3 sharpen() {
    vec4 base;
    vec3 tmp1, tmp2;
    tmp2 = vec3(0.0);
    base = texture2D(videoFrame, textureCoordinate);
    float delta = 0.0015625;
    for (highp int i = 0; i < 5; i++){
        for (highp int j = 0; j < 5; j++){
            float x, y;
            x = float(i - 2);
            y = float(j - 2);
            vec3 tmp = texture2D(videoFrame, textureCoordinate + vec2(delta * x, delta * y)).rgb;
            tmp2 = tmp2 + (tmp * 0.04);
        }
    }
    tmp1 = base.rgb * 2.0;
    return tmp1 - tmp2;
}


float opacityEye(float x, float y, vec2 cntr, float w, float h) {
   x = x - cntr.x;
   y = y - cntr.y;

   float funcf = x * x / (w * w) + y * y / (h * h);
   if (funcf <= 1.0) {
      return 0.5;
   } else if (funcf < 1.4) {
      return 1.25 * funcf - 0.75;
   } else {
      return 1.0;
   }
}

float opacity(float x, float y) {
   float w = abs(faceRect.x - faceRect.z) / 2.0;
   float h = abs(faceRect.y - faceRect.w) / 2.0;
   x = x - faceRect.x - w;
   y = y - faceRect.y - h;
   h = w * 20.0 / 15.0;
   float eyeW = 0.25 * w;
   float eyeH = eyeW / 2.0;
   float mouthW = 0.5 * w;
   float mouthH = mouthW / 2.0;

   float funcf = x * x / (w * w) + y * y / (h * h);
   if (funcf <= 1.0) {
      float opa = opacityEye(textureCoordinate.x, 1.0 - textureCoordinate.y, leftEye, eyeW, eyeH);
      if (opa < 1.0)
        return opa;
      opa = opacityEye(textureCoordinate.x, 1.0 - textureCoordinate.y,rightEye, eyeW, eyeH);
      if (opa < 1.0)
        return opa;
      opa = opacityEye(textureCoordinate.x, 1.0 - textureCoordinate.y, mouth, mouthW, mouthH);
      if (opa < 1.0)
        return opa;
      return 1.0;
   } else if (funcf < 1.4) {
      return 3.5 - 2.5 * funcf;
   } else {
      return 0.0;
   }
}



void main() {
    highp vec4 base;
    highp vec4 blend;

    base = texture2D(videoFrame, textureCoordinate);
    gl_FragColor.rgb = mild();
    gl_FragColor.rgb = mix(base.rgb, gl_FragColor.rgb, opacity(textureCoordinate.x, 1.0 - textureCoordinate.y));
}