varying highp vec2 textureCoordinate;
precision highp float;
uniform sampler2D videoFrame;
uniform sampler2D supTex1;

vec3 RGBToHSL(vec3 color)
{
 vec3 hsl;

 float fmin = min(min(color.r, color.g), color.b);
 float fmax = max(max(color.r, color.g), color.b);
 float delta = fmax - fmin;

 hsl.z = (fmax + fmin) / 2.0;

 if (delta == 0.0)
 {
  hsl.x = 0.0;
  hsl.y = 0.0;
 }
 else
 {
  if (hsl.z < 0.5)
   hsl.y = delta / (fmax + fmin);
  else
   hsl.y = delta / (2.0 - fmax - fmin);

  float deltaR = (((fmax - color.r) / 6.0) + (delta / 2.0)) / delta;
  float deltaG = (((fmax - color.g) / 6.0) + (delta / 2.0)) / delta;
  float deltaB = (((fmax - color.b) / 6.0) + (delta / 2.0)) / delta;

  if (color.r == fmax )
   hsl.x = deltaB - deltaG;
  else if (color.g == fmax)
   hsl.x = (1.0 / 3.0) + deltaR - deltaB;
  else if (color.b == fmax)
   hsl.x = (2.0 / 3.0) + deltaG - deltaR;

  if (hsl.x < 0.0)
   hsl.x += 1.0;
  else if (hsl.x > 1.0)
   hsl.x -= 1.0;
 }

 return hsl;
}

float HueToRGB(float f1, float f2, float hue) {
    if (hue < 0.0) {
        hue += 1.0;
    } else if (hue > 1.0) {
        hue -= 1.0;
    }
    float res;
    if (hue < 0.1666667) {
        res = f1 + (f2 - f1) * 6.0 * hue;
    } else if (hue < 0.5) {
        res = f2;
    } else if (hue < 0.666667) {
        res = f1 + (f2 - f1) * (0.666667 - hue) * 6.0;
    } else {
        res = f1;
    }
    return res;
}

vec3 HSLToRGB(vec3 hsl) {
    vec3 rgb;

    if (hsl.y == 0.0) {
        rgb = vec3(hsl.z);
    } else {
        float f2;

        if (hsl.z < 0.5)
            f2 = hsl.z * (1.0 + hsl.y);
        else
            f2 = (hsl.z + hsl.y) - (hsl.y * hsl.z);

        float f1 = 2.0 * hsl.z - f2;

        rgb.r = HueToRGB(f1, f2, hsl.x + (1.0 / 3.0));
        rgb.g = HueToRGB(f1, f2, hsl.x);
        rgb.b = HueToRGB(f1, f2, hsl.x - (1.0 / 3.0));
    }
    return rgb;
}

vec3 colorsCorrection(vec3 base) {
    highp vec2 lookup;

    base.r = base.r * 0.9 + 0.05;
    lookup.y = base.g;
    lookup.x = (base.r + floor(base.b * 15.0 + 0.50)) / 16.0;
    base.rgb = texture2D(supTex1, lookup).rgb;
    return base;
}

vec3 BlendColor(vec3 base, vec3 blend)
{
 vec3 baseHSL = RGBToHSL(base);
 vec3 blendHSL = RGBToHSL(blend);
 return HSLToRGB(vec3(blendHSL.r, blendHSL.g, baseHSL.b));
}

void main() {
    highp vec4 color;
    highp vec4 temp;
    color = texture2D(videoFrame, textureCoordinate);
    color.rgb = colorsCorrection(color.rgb);
    temp = vec4(0.921875, 0.96875, 0.91015625, 0.07);
    color.rgb = mix(color.rgb, vec3(max(temp.r, color.r), max(temp.g, color.g), max(temp.b, color.b)), temp.a);
    temp = vec4(0.2890625, 0.0, 0.82421875, 0.30);
    color.rgb = mix(color.rgb, BlendColor(color.rgb, temp.rgb), temp.a);
    temp = vec4(1.0, 0.91796875, 0.0078125, 0.06);
    color.rgb = mix(color.rgb, vec3((color.r < 0.5 ? (2.0 * color.r * temp.r) : (1.0 - 2.0 * (1.0 - color.r) * (1.0 - temp.r))), (color.g < 0.5 ? (2.0 * color.g * temp.g) : (1.0 - 2.0 * (1.0 - color.g) * (1.0 - temp.g))), (color.b < 0.5 ? (2.0 * color.b * temp.b) : (1.0 - 2.0 * (1.0 - color.b) * (1.0 - temp.b)))), temp.a);
    gl_FragColor = color;
}
