varying highp vec2 textureCoordinate;
precision highp float;
uniform sampler2D videoFrame;
uniform sampler2D supTex1;
uniform sampler2D supTex2;
uniform sampler2D supTex3;
uniform sampler2D supTex4;
uniform sampler2D supTex5;

vec3 RGBToHSL(vec3 color)
{
 vec3 hsl;

 float fmin = min(min(color.r, color.g), color.b);
 float fmax = max(max(color.r, color.g), color.b);
 float delta = fmax - fmin;

 hsl.z = (fmax + fmin) / 2.0;

 if (delta == 0.0)
 {
  hsl.x = 0.0;
  hsl.y = 0.0;
 }
 else
 {
  if (hsl.z < 0.5)
   hsl.y = delta / (fmax + fmin);
  else
   hsl.y = delta / (2.0 - fmax - fmin);

  float deltaR = (((fmax - color.r) / 6.0) + (delta / 2.0)) / delta;
  float deltaG = (((fmax - color.g) / 6.0) + (delta / 2.0)) / delta;
  float deltaB = (((fmax - color.b) / 6.0) + (delta / 2.0)) / delta;

  if (color.r == fmax )
   hsl.x = deltaB - deltaG;
  else if (color.g == fmax)
   hsl.x = (1.0 / 3.0) + deltaR - deltaB;
  else if (color.b == fmax)
   hsl.x = (2.0 / 3.0) + deltaG - deltaR;

  if (hsl.x < 0.0)
   hsl.x += 1.0;
  else if (hsl.x > 1.0)
   hsl.x -= 1.0;
 }

 return hsl;
}

float HueToRGB(float f1, float f2, float hue)
{
 if (hue < 0.0)
  hue += 1.0;
 else if (hue > 1.0)
  hue -= 1.0;
 float res;
 if ((6.0 * hue) < 1.0)
  res = f1 + (f2 - f1) * 6.0 * hue;
 else if ((2.0 * hue) < 1.0)
  res = f2;
 else if ((3.0 * hue) < 2.0)
  res = f1 + (f2 - f1) * ((2.0 / 3.0) - hue) * 6.0;
 else
  res = f1;
 return res;
}

vec3 HSLToRGB(vec3 hsl)
{
 vec3 rgb;

 if (hsl.y == 0.0)
  rgb = vec3(hsl.z);
 else
 {
  float f2;

  if (hsl.z < 0.5)
   f2 = hsl.z * (1.0 + hsl.y);
  else
   f2 = (hsl.z + hsl.y) - (hsl.y * hsl.z);

  float f1 = 2.0 * hsl.z - f2;

  rgb.r = HueToRGB(f1, f2, hsl.x + (1.0/3.0));
  rgb.g = HueToRGB(f1, f2, hsl.x);
  rgb.b= HueToRGB(f1, f2, hsl.x - (1.0/3.0));
 }

 return rgb;
}

vec3 BlendLuminosity(vec3 base, vec3 blend)
{
 vec3 baseHSL = RGBToHSL(base);
 return HSLToRGB(vec3(baseHSL.r, baseHSL.g, RGBToHSL(blend).b));
}

vec3 colorsCorrection(vec3 base) {
    highp vec2 lookup;
    base.r = base.r * 0.9 + 0.05;
    lookup.y = base.g;
    lookup.x = (base.r + floor(base.b * 15.0 + 0.50)) / 16.0;
    base.rgb = texture2D(supTex1, lookup).rgb;
    return base;
}

void main() {
    highp vec4 color;
    highp vec4 c2;
    highp vec4 c3;
    highp vec4 c4;
    highp vec4 c5;
    color = texture2D(videoFrame, textureCoordinate);
    color.rgb = colorsCorrection(color.rgb);
    c2 = texture2D(supTex2, textureCoordinate);
    color.rgb = mix(color.rgb, vec3((c2.r < 0.5 ? (2.0 * c2.r * color.r) : (1.0 - 2.0 * (1.0 - c2.r) * (1.0 - color.r))), (c2.g < 0.5 ? (2.0 * c2.g * color.g) : (1.0 - 2.0 * (1.0 - c2.g) * (1.0 - color.g))), (c2.b < 0.5 ? (2.0 * c2.b * color.b) : (1.0 - 2.0 * (1.0 - c2.b) * (1.0 - color.b)))), c2.a);
    c3 = texture2D(supTex3, textureCoordinate);
    color.rgb = mix(color.rgb, BlendLuminosity(color.rgb, c3.rgb), c3.a);
    c4 = texture2D(supTex4, textureCoordinate);
    color.rgb = mix(color.rgb, vec3(max(min(color.r + 2.0 * c4.r - 1.0, 1.0), 0.0), max(min(color.g + 2.0 * c4.g - 1.0, 1.0), 0.0), max(min(color.b + 2.0 * c4.b - 1.0, 1.0), 0.0)), c4.a);
    c5 = texture2D(supTex5, textureCoordinate);
    color.rgb = mix(color.rgb, vec3((c5.r < 0.5 ? (2.0 * c5.r * color.r) : (1.0 - 2.0 * (1.0 - c5.r) * (1.0 - color.r))), (c5.g < 0.5 ? (2.0 * c5.g * color.g) : (1.0 - 2.0 * (1.0 - c5.g) * (1.0 - color.g))), (c5.b < 0.5 ? (2.0 * c5.b * color.b) : (1.0 - 2.0 * (1.0 - c5.b) * (1.0 - color.b)))), c5.a);
    gl_FragColor = color;
}
