varying highp vec2 textureCoordinate;
precision highp float;
uniform highp vec4 boundingParams;
uniform sampler2D videoFrame;

float whitening(float d, float r) {
    float w;
    w = (d - r) * 2.0;
    if (w > 0.4) {
        w = 0.4;
    }
    w *= boundingParams.w;
    return w;
}

void main() {
    vec4 base = texture2D(videoFrame, textureCoordinate);
    float r = boundingParams.x / 2.0 + 0.1;
    vec2 center = vec2(boundingParams.y, boundingParams.z);
    float d = distance(textureCoordinate, center);
    if (d > r) {
        float delta = 0.02 * (d - r);
        vec4 sample = vec4(0.0);
        if (delta > 0.0055) {
            delta = 0.0055;
            sample = vec4(1.0);
        }

        sample = base * 0.32258;
        sample += texture2D(videoFrame, textureCoordinate + vec2(-delta * 2.0, 0.0)) * 0.048387;
        sample += texture2D(videoFrame, textureCoordinate + vec2(-delta, 0.0)) * 0.120965;
        sample += texture2D(videoFrame, textureCoordinate + vec2(delta, 0.0)) * 0.120965;
        sample += texture2D(videoFrame, textureCoordinate + vec2(delta * 2.0, 0.0)) * 0.048387;
        sample += texture2D(videoFrame, textureCoordinate + vec2(0.0, -delta * 2.0)) * 0.048387;
        sample += texture2D(videoFrame, textureCoordinate + vec2(0.0, delta)) * 0.120965;
        sample += texture2D(videoFrame, textureCoordinate + vec2(0.0, delta)) * 0.120965;
        sample += texture2D(videoFrame, textureCoordinate + vec2(0.0, delta * 2.0)) * 0.048387;
        sample = sample + whitening(d, r);
        gl_FragColor = vec4(sample.rgb, 1.0);
    } else {
        gl_FragColor = base;
    }
}