uniform sampler2D videoFrame;
uniform sampler2D supTex1;
uniform vec3 addParams;
varying highp vec2 textureCoordinate;



const float filterStrength = 0.5;

const float pointRadius = 0.75;

void main() {
    vec2 points[3] = vec2[3](vec2(0.25, 0.25), vec2(0.75, 0.25), vec2(0.5, 0.75));
    vec3 colors[3] = vec3[3](vec3(0.827450980392, 0.890196078431, 0.592156862745), vec3(0.552941176471, 0.803921568627, 0.756862745098), vec3(0.921568627451, 0.43137254902, 0.266666666667));
    lowp float distance1 = distance(textureCoordinate, points[0]) / pointRadius;
    lowp float distance2 = distance(textureCoordinate, points[1]) / pointRadius;
    lowp float distance3 = distance(textureCoordinate, points[2]) / pointRadius;
    lowp float textureInvert = 0.0;
    lowp float textureStrength = 0.0;
    lowp float textureOutside = 0.0;
    float overlayIntensity[3] = float[3](0.3, 0.3, 0.3);
    float multiplyIntensity[3] = float[3](0.2, 0.2, 0.2);
    float contrastIntensity[3] = float[3](0.3, 0.3, 0.3);
    float brightnessIntensity[3] = float[3](0.0, 0.0, 0.0);
    float additiveIntensity[3] = float[3](0.0, 0.0, 0.0);
    float differenceIntensity[3] = float[3](0.0, 0.0, 0.0);
    float saturationIntensity[3] = float[3](0.0, 0.0, 0.0);
    colors[0].r = addParams[0];
    colors[1].g = addParams[1];
    colors[2].b = addParams[2];


    textureStrength = addParams[0];
    overlayIntensity[0] = addParams[0];
    overlayIntensity[1] = addParams[1];
    overlayIntensity[2] = addParams[2];

    contrastIntensity[1] = addParams[0];
    contrastIntensity[2] = addParams[1];
    contrastIntensity[0] = addParams[2];

    multiplyIntensity[2] = addParams[0];
    multiplyIntensity[0] = addParams[1];
    multiplyIntensity[1] = addParams[2];

    distance1 = 1.0 - smoothstep(0.0, 1.0, distance1);
    distance2 = 1.0 - smoothstep(0.0, 1.0, distance2);
    distance3 = 1.0 - smoothstep(0.0, 1.0, distance3);
    // calculate the local strength using the texture and the filterStrengh

    // start with the texture color
    lowp float textureValue = texture2D(supTex1, textureCoordinate).r;

    textureValue = mix(textureValue, 1.0 - textureValue, textureInvert);

    lowp float localFilterStrength = textureValue * (1.0 - textureOutside) * textureStrength;

    // invert if necessary

    // mix with strength
    localFilterStrength = (localFilterStrength - (0.5 * textureStrength)) * filterStrength + filterStrength;


    lowp float colorBalance = clamp(distance1 + distance2 + distance3, 0.01, 1.0); // don't let this get as low as (divide by 0 errors)
    lowp vec3 localColor = vec3((colors[0] * distance1) + (colors[1] * distance2) + (colors[2] * distance3));
    lowp vec3 balancedLocalColor = localColor / colorBalance;

    mediump vec3 imageTexelColor = texture2D(videoFrame, textureCoordinate).rgb;
    lowp vec3 originalImageColor = imageTexelColor;

    // apply the overlay
    lowp float localOverlayIntensity = (overlayIntensity[0] * distance1) + (overlayIntensity[1] * distance2) + (overlayIntensity[2] * distance3);
    imageTexelColor = mix(imageTexelColor, localColor, localOverlayIntensity * localFilterStrength);

    // apply the difference
    lowp float localDifferenceIntensity = (differenceIntensity[0] * distance1) + (differenceIntensity[1] * distance2) + (differenceIntensity[2] * distance3);
    localDifferenceIntensity = clamp(localDifferenceIntensity * localFilterStrength, -1.0, 1.0);

    imageTexelColor = abs(imageTexelColor - (localColor * localDifferenceIntensity));

    // apply the brightness
    lowp float localBrightnessIntensity = (brightnessIntensity[0] * distance1) + (brightnessIntensity[1] * distance2) + (brightnessIntensity[2] * distance3);
    localBrightnessIntensity = clamp(localBrightnessIntensity * localFilterStrength, -1.0, 1.0);

    imageTexelColor = imageTexelColor + (localBrightnessIntensity * localColor);

    lowp float localContrastIntensity = (contrastIntensity[0] * distance1) + (contrastIntensity[1] * distance2) + (contrastIntensity[2] * distance3);
    localContrastIntensity = clamp(localContrastIntensity * localFilterStrength, -1.0, 1.0);

    // apply the contrast
    imageTexelColor = (imageTexelColor -  0.5) * (1.0 + (localContrastIntensity * 3.0)) + 0.5;
    imageTexelColor = mix(imageTexelColor, imageTexelColor * balancedLocalColor, localContrastIntensity);


    // apply the additive blend

    lowp float localAdditiveIntensity = (additiveIntensity[0] * distance1) + (additiveIntensity[1] * distance2) + (additiveIntensity[2] * distance3);
    localAdditiveIntensity = clamp(localAdditiveIntensity * localFilterStrength, -1.0, 1.0);

    highp vec3 blendedTexelColor = imageTexelColor + (imageTexelColor * localAdditiveIntensity * 2.0 * localColor);
    imageTexelColor = mix(imageTexelColor, blendedTexelColor, localAdditiveIntensity);



    // apply the saturation blend
    lowp float localSaturationIntensity = (saturationIntensity[0] * distance1) + (saturationIntensity[1] * distance2) + (saturationIntensity[2] * distance3);
    localSaturationIntensity = clamp(localSaturationIntensity * localFilterStrength, -1.0, 1.0);

    highp vec3 grayXfer = vec3(0.3, 0.59, 0.11);
    highp vec3 gray = vec3(dot(grayXfer, imageTexelColor.rgb));
    imageTexelColor = mix(imageTexelColor, gray  * balancedLocalColor, localSaturationIntensity);


    lowp float localMultiplyIntensity = (multiplyIntensity[0] * distance1) + (multiplyIntensity[1] * distance2) + (multiplyIntensity[2] * distance3);
    localMultiplyIntensity = clamp(localMultiplyIntensity * localFilterStrength, -1.0, 1.0);

    imageTexelColor =  mix(imageTexelColor, imageTexelColor * balancedLocalColor, localMultiplyIntensity);


    // apply the texture outside value as overlay
    highp vec3 br = clamp(sign(imageTexelColor.rgb - vec3(0.5)), vec3(0.0), vec3(1.0));
    highp vec3 multiply = 2.0 * imageTexelColor.rgb * vec3(textureValue);
    highp vec3 screen = vec3(1.0) - 2.0 * (vec3(1.0) - imageTexelColor.rgb) * (vec3(1.0) - vec3(textureValue));
    highp vec3 overlay = mix(multiply, screen, br);
    imageTexelColor = mix(imageTexelColor.rgb, overlay, textureOutside * textureStrength * filterStrength * 0.75);

    imageTexelColor = clamp(imageTexelColor, 0.0, 1.0);

    gl_FragColor = vec4(imageTexelColor, 1.0);
}